# Generated by roxygen2: do not edit by hand

export(rMASC)
importFrom(Rcpp,evalCpp)
importFrom(stats,pnorm)
importFrom(stats,qnorm)
importFrom(stats,rbeta)
importFrom(stats,rmultinom)
importFrom(stats,rnorm)
useDynLib(masc, .registration = TRUE)
